package com.simualteSmack;

import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import com.simualteSmack.model.Packet;
/**      
 * @date 2011-7-31  
 * @author zhanghaitao     
 * @version 1.0 
 *     
 */
public abstract class Connection {
	protected Reader reader;
	protected Writer writer;
	protected final ConnectionConfiguration config;

	/**
	 * List of PacketListeners that will be notified when a new packet was
	 * received.
	 */
	protected final Map<PacketListener, ListenerWrapper> recvListeners = new ConcurrentHashMap<PacketListener, ListenerWrapper>();
	/**
	 * A collection of PacketCollectors which collects packets for a specified
	 * filter and perform blocking and polling operations on the result queue.
	 */
	protected final Collection<PacketCollector> collectors = new ConcurrentLinkedQueue<PacketCollector>();

	protected Connection(ConnectionConfiguration configuration) {
		config = configuration;
	}

	/**
	 * The SmackDebugger allows to log and debug XML traffic.
	 */
	protected SmackDebugger debugger = null;

	/**
	 * A wrapper class to associate a packet filter with a listener.
	 */
	protected static class ListenerWrapper {

		private PacketListener packetListener;
		private PacketFilter packetFilter;

		/**
		 * Create a class which associates a packet filter with a listener.
		 * 
		 * @param packetListener
		 *            the packet listener.
		 * @param packetFilter
		 *            the associated filter or null if it listen for all
		 *            packets.
		 */
		public ListenerWrapper(PacketListener packetListener,
				PacketFilter packetFilter) {
			this.packetListener = packetListener;
			this.packetFilter = packetFilter;
		}

		/**
		 * Notify and process the packet listener if the filter matches the
		 * packet.
		 * 
		 * @param packet
		 *            the packet which was sent or received.
		 */
		public void notifyListener(Packet packet) {
			if (packetFilter == null || packetFilter.accept(packet)) {
				packetListener.processPacket(packet);
			}
		}
	}

	/**
	 * Creates a new packet collector for this connection. A packet filter
	 * determines which packets will be accumulated by the collector. A
	 * PacketCollector is more suitable to use than a {@link PacketListener}
	 * when you need to wait for a specific result.
	 * 
	 * @param packetFilter
	 *            the packet filter to use.
	 * @return a new packet collector.
	 */
	public PacketCollector createPacketCollector(PacketFilter packetFilter) {
		PacketCollector collector = new PacketCollector(this, packetFilter);
		// Add the collector to the list of active collectors.
		collectors.add(collector);
		return collector;
	}
	
//	public PacketCollector addPacketCollector(PacketCollector packetCollector) {
//		// Add the collector to the list of active collectors.
//		collectors.add(packetCollector);
//		return collector;
//	}

	/**
	 * Get the collection of all packet collectors for this connection.
	 * 
	 * @return a collection of packet collectors for this connection.
	 */
	protected Collection<PacketCollector> getPacketCollectors() {
		return collectors;
	}

	/**
	 * Remove a packet collector of this connection.
	 * 
	 * @param collector
	 *            a packet collectors which was created for this connection.
	 */
	protected void removePacketCollector(PacketCollector collector) {
		collectors.remove(collector);
	}

	/**
	 * Registers a packet listener with this connection. A packet filter
	 * determines which packets will be delivered to the listener. If the same
	 * packet listener is added again with a different filter, only the new
	 * filter will be used.
	 * 
	 * @param packetListener
	 *            the packet listener to notify of new received packets.
	 * @param packetFilter
	 *            the packet filter to use.
	 */
	public void addPacketListener(PacketListener packetListener,
			PacketFilter packetFilter) {
		if (packetListener == null) {
			throw new NullPointerException("Packet listener is null.");
		}
		ListenerWrapper wrapper = new ListenerWrapper(packetListener,
				packetFilter);
		recvListeners.put(packetListener, wrapper);
	}

	/**
	 * Removes a packet listener for received packets from this connection.
	 * 
	 * @param packetListener
	 *            the packet listener to remove.
	 */
	public void removePacketListener(PacketListener packetListener) {
		recvListeners.remove(packetListener);
	}

	/**
	 * Get a map of all packet listeners for received packets of this
	 * connection.
	 * 
	 * @return a map of all packet listeners for received packets.
	 */
	protected Map<PacketListener, ListenerWrapper> getPacketListeners() {
		return recvListeners;
	}

	protected void initDebugger() {
		Class<?> debuggerClass = null;
		try {
			debuggerClass = Class.forName("com.simualteSmack.ConsoleDebugger");

			Constructor<?> constructor = debuggerClass.getConstructor(
					Connection.class, Writer.class, Reader.class);
			debugger = (SmackDebugger) constructor.newInstance(this, writer,
					reader);
			reader = debugger.getReader();
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (Exception e) {
			throw new IllegalArgumentException(
					"Can't initialize the configured debugger!", e);
		}
	}
}
