package com.simualteSmack;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

import com.simualteSmack.model.Packet;
/**      
 * @date 2011-7-31  
 * @author zhanghaitao     
 * @version 1.0 
 *     
 */
public class PacketCollector {
	/**
	 * Max number of packets that any one collector can hold. After the max is
	 * reached, older packets will be automatically dropped from the queue as
	 * new packets are added.
	 */
	private static final int MAX_PACKETS = 65536;

	private PacketFilter packetFilter;
	private LinkedBlockingQueue<Packet> resultQueue;
	private Connection connection;

	private boolean cancelled = false;

	/**
	 * Creates a new packet collector. If the packet filter is <tt>null</tt>,
	 * then all packets will match this collector.
	 * 
	 * @param connection
	 *            the connection the collector is tied to.
	 * @param packetFilter
	 *            determines which packets will be returned by this collector.
	 */
	protected PacketCollector(Connection connection, PacketFilter packetFilter) {
		this.connection = connection;
		this.packetFilter = packetFilter;
		this.resultQueue = new LinkedBlockingQueue<Packet>(MAX_PACKETS);
	}

	/**
	 * Returns the packet filter associated with this packet collector. The
	 * packet filter is used to determine what packets are queued as results.
	 * 
	 * @return the packet filter.
	 */
	public PacketFilter getPacketFilter() {
		return packetFilter;
	}

	/**
	 * Polls to see if a packet is currently available and returns it, or
	 * immediately returns <tt>null</tt> if no packets are currently in the
	 * result queue.
	 * 
	 * @return the next packet result, or <tt>null</tt> if there are no more
	 *         results.
	 */
	public Packet pollResult() {
		return resultQueue.poll();
	}

	/**
	 * Returns the next available packet. The method call will block (not
	 * return) until a packet is available.
	 * 
	 * @return the next available packet.
	 */
	public Packet nextResult() {
		while (true) {
			try {
				return resultQueue.take();
			} catch (InterruptedException e) { /* ignore */
			}
		}
	}

	/**
	 * Returns the next available packet. The method call will block (not
	 * return) until a packet is available or the <tt>timeout</tt> has elapased.
	 * If the timeout elapses without a result, <tt>null</tt> will be returned.
	 * 
	 * @param timeout
	 *            the amount of time to wait for the next packet (in
	 *            milleseconds).
	 * @return the next available packet.
	 */
	public Packet nextResult(long timeout) {
		long endTime = System.currentTimeMillis() + timeout;
		System.out.println("nextResult");
		do {
			try {
				return resultQueue.poll(timeout, TimeUnit.MILLISECONDS);
			} catch (InterruptedException e) { /* ignore */
			}
		} while (System.currentTimeMillis() < endTime);
		return null;
	}

	/**
	 * Processes a packet to see if it meets the criteria for this packet
	 * collector. If so, the packet is added to the result queue.
	 * 
	 * @param packet
	 *            the packet to process.
	 */
	protected synchronized void processPacket(Packet packet) {
		System.out.println("PacketCollector---processPacket");
		if (packet == null) {
			return;
		}
		if (packetFilter == null || packetFilter.accept(packet)) {
			while (!resultQueue.offer(packet)) {
				resultQueue.poll();
			}
		}
	}

	/**
	 * Explicitly cancels the packet collector so that no more results are
	 * queued up. Once a packet collector has been cancelled, it cannot be
	 * re-enabled. Instead, a new packet collector must be created.
	 */
	public void cancel() {
		// If the packet collector has already been cancelled, do nothing.
		if (!cancelled) {
			connection.removePacketCollector(this);
			cancelled = true;
		}
	}
}
